==> content_management-0.0.1.gem: app/models/content_management/content.rb <==
# coding: utf-8
require 'uri'
require 'net/http'

module ContentManagement
  class Content < ActiveRecord::Base
    self.table_name = 'cm_contents'

    belongs_to :owner, :polymorphic => true
    belongs_to :content_format, :class_name => '::ContentManagement::ContentFormat', :validate => true
    belongs_to :content_category, :class_name => '::ContentManagement::ContentCategory', :validate => true
    belongs_to :content_type, :class_name => '::ContentManagement::ContentType', :validate => true
    belongs_to :content_audience, :class_name => '::ContentManagement::ContentAudience', :validate => true
    belongs_to :content_status, :class_name => '::ContentManagement::ContentStatus', :validate => true

    #has_many :comments, :class_name => "Content", :dependent => :destroy
    has_many :comments, :class_name => '::ContentManagement::Comment', :dependent => :destroy

    has_many :trackbacks, :class_name => '::ContentManagement::Trackback', :dependent => :destroy

    #acts_as_taggable
    has_and_belongs_to_many :tags, :class_name => '::ContentManagement::Tag', :join_table => 'cm_contents_tags'

    #---------------------------------------------------------------------------

    attr_accessible :version, :title, :permalink
    attr_accessible :excerpt, :body, :extension
    attr_accessible :allow_comments_until, :comments_count, :allow_hit_counts, :hit_count, :approved
    attr_accessible :published_at, :expires_at
    attr_accessible :ip_address, :keywords, :referrer, :text_filter, :user_agent
    attr_accessible :owner, :owner_id
    attr_accessible :content_format, :content_format_id
    attr_accessible :content_category, :content_category_id
    attr_accessible :content_type, :content_type_id
    attr_accessible :content_audience, :content_audience_id
    attr_accessible :content_status, :content_status_id

    #---------------------------------------------------------------------------

    validates :title, :presence => true
    validates :permalink, :presence => true
    #validates :owner, :presence => true
    validates :content_format, :presence => true
    validates :content_type, :presence => true
    validates :content_audience, :presence => true
    validates :content_category, :presence => true

    #---------------------------------------------------------------------------

    scope :static_pages, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::StaticPage))
    }

    scope :articles, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::Article))
    }

    scope :blogs, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::Blog))
    }

    scope :faqs, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::FAQ))
    }

    scope :created_in_year, lambda { |year|
      where("created_at IS NOT NULL AND created_at >= ? AND created_at <= ?",
        Date.new(year.to_i,1,1).at_beginning_of_year.to_s, Date.new(year.to_i,1,1).at_end_of_year.to_s )
    }

    scope :created_in_month, lambda { |year, month|
      where("created_at IS NOT NULL AND created_at >= ? AND created_at <= ?",
        Date.new(year.to_i,month.to_i,1).at_beginning_of_month.to_s, Date.new(year.to_i,month.to_i,1).at_end_of_month.to_s )
    }

    scope :created_on, lambda { |year, month, day|
      where("created_at IS NOT NULL AND created_at >= ? AND created_at <= ?",
        Date.new(year.to_i,month.to_i,day.to_i).at_beginning_of_day.to_s, Date.new(year.to_i,month.to_i,day.to_i).tomorrow.at_beginning_of_day.to_s)
    }

    #---------------------------------------------------------------------------

    before_save :convert_keywords_to_tags

    #---------------------------------------------------------------------------

    def convert_keywords_to_tags
      Content.transaction do
        tags.clear
        tags << keywords.to_s.scan(/\"([\w ]+)\"|(\w+)/).flatten.compact.uniq.map do |tagword|
          Tag.get(tagword)
        end
      end
    end

    #---------------------------------------------------------------------------
    # See: https://raw.github.com/fdv/typo/master/app/models/article.rb

    accents = {
      ['á','à','â','ä','ã','Ã','Ä','Â','À'] => 'a',
      ['é','è','ê','ë','Ë','É','È','Ê'] => 'e',
      ['í','ì','î','ï','I','Î','Ì'] => 'i',
      ['ó','ò','ô','ö','õ','Õ','Ö','Ô','Ò'] => 'o',
      ['œ'] => 'oe',
      ['ß'] => 'ss',
      ['ú','ù','û','ü','U','Û','Ù'] => 'u',
      ['ç','Ç'] => 'c'
    }

    FROM, TO = accents.inject(['','']) { |o,(k,v)|
      o[0] << k * '';
      o[1] << v * k.size
      o
    }

    def stripped_title
      self.title.tr(FROM, TO).gsub(/<[^>]*>/, '').to_url
    end

    def year_url
      published_at.year.to_s
    end

    def month_url
      sprintf("%.2d", published_at.month)
    end

    def day_url
      sprintf("%.2d", published_at.day)
    end

    def title_url
      URI.encode(permalink.to_s)
    end

    def permalink_url_options(nesting = false)
      format_url = content.permalink_format.dup
      format_url.gsub!('%year%', year_url)
      format_url.gsub!('%month%', month_url)
      format_url.gsub!('%day%', day_url)
      format_url.gsub!('%title%', title_url)
      if format_url[0,1] == '/'
        format_url[1..-1]
      else
        format_url
      end
    end

    def permalink_url(anchor=nil, only_path=false)
      @cached_permalink_url ||= {}

      @cached_permalink_url["#{anchor}#{only_path}"] ||= content.url_for(permalink_url_options, :anchor => anchor, :only_path => only_path)
    end

    def param_array
      unless published_at.nil?
        @param_array ||= [
          published_at.year,
          sprintf('%.2d', published_at.month),
          sprintf('%.2d', published_at.day),
          permalink
        ].tap do |params|
          this = self
          k = class << params; self; end
          k.send(:define_method, :to_s) { params[-1] }
        end
      end
    end

    #def to_param
    #  param_array
    #end

    def trackback_url
      blog.url_for("trackbacks?article_id=#{self.id}", :only_path => false)
    end

    def permalink_by_format(format=nil)
      if format.nil?
        permalink_url
      elsif format.to_sym == :rss
        feed_url(:rss)
      elsif format.to_sym == :atom
        feed_url(:atom)
      else
        raise UnSupportedFormat
      end
    end

    def comment_url
      blog.url_for("comments?article_id=#{self.id}", :only_path => false)
    end

    def preview_comment_url
      blog.url_for("comments/preview?article_id=#{self.id}", :only_path => false)
    end

    def feed_url(format = :rss20)
      format_extension = format.to_s.gsub(/\d/,'')
      permalink_url + ".#{format_extension}"
    end

    def edit_url
      blog.url_for(:controller => "/admin/content", :action =>"edit", :id => id)
    end

    def delete_url
      blog.url_for(:controller => "/admin/content", :action =>"destroy", :id => id)
    end

    def html_urls
      urls = Array.new
      html.gsub(/<a\s+[^>]*>/) do |tag|
        if(tag =~ /\bhref=(["']?)([^ >"]+)\1/)
          urls.push($2.strip)
        end
      end

      urls.uniq
    end

    def next
      self.class.find(:first, :conditions => ['published_at > ?', published_at],
        :order => 'published_at asc')
    end

    def previous
      self.class.find(:first, :conditions => ['published_at < ?', published_at],
        :order => 'published_at desc')
    end

    # Count articles on a certain date
    def self.count_by_date(year, month = nil, day = nil, limit = nil)
      if !year.blank?
        count(:conditions => { :published_at => time_delta(year, month, day),
            :published => true })
      else
        count(:conditions => { :published => true })
      end
    end

    # Finds one article which was posted on a certain date and matches the supplied dashed-title
    # params is a Hash
    #    def self.find_by_permalink(params)
    #      date_range = self.time_delta(params[:year], params[:month], params[:day])
    #
    #      req_params = {}
    #      req_params[:permalink] = params[:title] if params[:title]
    #      req_params[:published_at] = date_range if date_range
    #
    #      return nil if req_params.empty? # no search if no params send
    #
    #      article = find_published(:first, :conditions => req_params)
    #      return article if article
    #
    #      if params[:title]
    #        req_params[:permalink] = CGI.escape(params[:title])
    #        article = find_published(:first, :conditions => req_params)
    #        return article if article
    #      end
    #
    #      raise ActiveRecord::RecordNotFound
    #    end

    def self.find_by_params_hash(params = {})
      params[:title] ||= params[:article_id]
      find_by_permalink(params)
    end

    # Fulltext searches the body of published articles
    def self.search(query, args={})
      query_s = query.to_s.strip
      if !query_s.empty? && args.empty?
        ::ContentManagement::Content.searchstring(query)
      elsif !query_s.empty? && !args.empty?
        ::ContentManagement::Content.searchstring(query).paginate(args)
      else
        []
      end
    end

    def add_comment(params)
      comments.build(params)
    end

    #---------------------------------------------------------------------------

    def type_is(content_type)
      self.content_type == ::ContentManagement::ContentType.find_by_name(content_type)
    end

    def format_is(content_format)
      self.content_format == ::ContentManagement::ContentFormat.find_by_name(content_format)
    end

    def published?
      !published_at.nil?
    end

    def pending?
      !published? || Time.now.utc < published_at
    end

    def status
      pending? ? :pending : :published
    end

    #---------------------------------------------------------------------------

    private

    def self.time_delta(year = nil, month = nil, day = nil)
      return nil if year.nil? && month.nil? && day.nil?
      from = Time.utc(year, month || 1, day || 1)

      to = from.next_year
      to = from.next_month unless month.blank?
      to = from + 1.day unless day.blank?
      to = to - 1 # pull off 1 second so we don't overlap onto the next day
      return from..to
    end

  end
end

==> content_management-0.1.0.gem: app/models/content_management/content.rb <==
# coding: utf-8
require 'uri'
require 'net/http'

module ContentManagement
  class Content < ActiveRecord::Base
    self.table_name = 'cm_contents'

    belongs_to :owner, :polymorphic => true
    belongs_to :content_format, :class_name => '::ContentManagement::ContentFormat', :validate => true
    belongs_to :content_category, :class_name => '::ContentManagement::ContentCategory', :validate => true
    belongs_to :content_type, :class_name => '::ContentManagement::ContentType', :validate => true
    belongs_to :content_audience, :class_name => '::ContentManagement::ContentAudience', :validate => true
    belongs_to :content_status, :class_name => '::ContentManagement::ContentStatus', :validate => true

    #has_many :comments, :class_name => "Content", :dependent => :destroy
    has_many :comments, :class_name => '::ContentManagement::Comment', :dependent => :destroy

    has_many :trackbacks, :class_name => '::ContentManagement::Trackback', :dependent => :destroy

    #acts_as_taggable
    has_and_belongs_to_many :tags, :class_name => '::ContentManagement::Tag', :join_table => 'cm_contents_tags'

    #---------------------------------------------------------------------------

    #attr_accessible :version, :title, :permalink
    #attr_accessible :excerpt, :body, :extension
    #attr_accessible :allow_comments_until, :comments_count, :allow_hit_counts, :hit_count, :approved
    #attr_accessible :published_at, :expires_at
    #attr_accessible :ip_address, :keywords, :referrer, :text_filter, :user_agent
    #attr_accessible :owner, :owner_id
    #attr_accessible :content_format, :content_format_id
    #attr_accessible :content_category, :content_category_id
    #attr_accessible :content_type, :content_type_id
    #attr_accessible :content_audience, :content_audience_id
    #attr_accessible :content_status, :content_status_id

    #---------------------------------------------------------------------------

    validates :title, :presence => true
    validates :permalink, :presence => true
    #validates :owner, :presence => true
    validates :content_format, :presence => true
    validates :content_type, :presence => true
    validates :content_audience, :presence => true
    validates :content_category, :presence => true

    #---------------------------------------------------------------------------

    scope :static_pages, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::StaticPage))
    }

    scope :articles, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::Article))
    }

    scope :blogs, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::Blog))
    }

    scope :faqs, lambda {
      where(:content_type => ::ContentManagement::ContentType.find_by_name(::ContentManagement::ContentType::FAQ))
    }

    scope :created_in_year, lambda { |year|
      where("created_at IS NOT NULL AND created_at >= ? AND created_at <= ?",
        Date.new(year.to_i,1,1).at_beginning_of_year.to_s, Date.new(year.to_i,1,1).at_end_of_year.to_s )
    }

    scope :created_in_month, lambda { |year, month|
      where("created_at IS NOT NULL AND created_at >= ? AND created_at <= ?",
        Date.new(year.to_i,month.to_i,1).at_beginning_of_month.to_s, Date.new(year.to_i,month.to_i,1).at_end_of_month.to_s )
    }

    scope :created_on, lambda { |year, month, day|
      where("created_at IS NOT NULL AND created_at >= ? AND created_at <= ?",
        Date.new(year.to_i,month.to_i,day.to_i).at_beginning_of_day.to_s, Date.new(year.to_i,month.to_i,day.to_i).tomorrow.at_beginning_of_day.to_s)
    }

    #---------------------------------------------------------------------------

    before_save :convert_keywords_to_tags

    #---------------------------------------------------------------------------

    def convert_keywords_to_tags
      Content.transaction do
        tags.clear
        tags << keywords.to_s.scan(/\"([\w ]+)\"|(\w+)/).flatten.compact.uniq.map do |tagword|
          Tag.get(tagword)
        end
      end
    end

    #---------------------------------------------------------------------------
    # See: https://raw.github.com/fdv/typo/master/app/models/article.rb

    accents = {
      ['á','à','â','ä','ã','Ã','Ä','Â','À'] => 'a',
      ['é','è','ê','ë','Ë','É','È','Ê'] => 'e',
      ['í','ì','î','ï','I','Î','Ì'] => 'i',
      ['ó','ò','ô','ö','õ','Õ','Ö','Ô','Ò'] => 'o',
      ['œ'] => 'oe',
      ['ß'] => 'ss',
      ['ú','ù','û','ü','U','Û','Ù'] => 'u',
      ['ç','Ç'] => 'c'
    }

    FROM, TO = accents.inject(['','']) { |o,(k,v)|
      o[0] << k * '';
      o[1] << v * k.size
      o
    }

    def stripped_title
      self.title.tr(FROM, TO).gsub(/<[^>]*>/, '').to_url
    end

    def year_url
      published_at.year.to_s
    end

    def month_url
      sprintf("%.2d", published_at.month)
    end

    def day_url
      sprintf("%.2d", published_at.day)
    end

    def title_url
      URI.encode(permalink.to_s)
    end

    def permalink_url_options(nesting = false)
      format_url = content.permalink_format.dup
      format_url.gsub!('%year%', year_url)
      format_url.gsub!('%month%', month_url)
      format_url.gsub!('%day%', day_url)
      format_url.gsub!('%title%', title_url)
      if format_url[0,1] == '/'
        format_url[1..-1]
      else
        format_url
      end
    end

    def permalink_url(anchor=nil, only_path=false)
      @cached_permalink_url ||= {}

      @cached_permalink_url["#{anchor}#{only_path}"] ||= content.url_for(permalink_url_options, :anchor => anchor, :only_path => only_path)
    end

    def param_array
      unless published_at.nil?
        @param_array ||= [
          published_at.year,
          sprintf('%.2d', published_at.month),
          sprintf('%.2d', published_at.day),
          permalink
        ].tap do |params|
          this = self
          k = class << params; self; end
          k.send(:define_method, :to_s) { params[-1] }
        end
      end
    end

    #def to_param
    #  param_array
    #end

    def trackback_url
      blog.url_for("trackbacks?article_id=#{self.id}", :only_path => false)
    end

    def permalink_by_format(format=nil)
      if format.nil?
        permalink_url
      elsif format.to_sym == :rss
        feed_url(:rss)
      elsif format.to_sym == :atom
        feed_url(:atom)
      else
        raise UnSupportedFormat
      end
    end

    def comment_url
      blog.url_for("comments?article_id=#{self.id}", :only_path => false)
    end

    def preview_comment_url
      blog.url_for("comments/preview?article_id=#{self.id}", :only_path => false)
    end

    def feed_url(format = :rss20)
      format_extension = format.to_s.gsub(/\d/,'')
      permalink_url + ".#{format_extension}"
    end

    def edit_url
      blog.url_for(:controller => "/admin/content", :action =>"edit", :id => id)
    end

    def delete_url
      blog.url_for(:controller => "/admin/content", :action =>"destroy", :id => id)
    end

    def html_urls
      urls = Array.new
      html.gsub(/<a\s+[^>]*>/) do |tag|
        if(tag =~ /\bhref=(["']?)([^ >"]+)\1/)
          urls.push($2.strip)
        end
      end

      urls.uniq
    end

    def next
      self.class.find(:first, :conditions => ['published_at > ?', published_at],
        :order => 'published_at asc')
    end

    def previous
      self.class.find(:first, :conditions => ['published_at < ?', published_at],
        :order => 'published_at desc')
    end

    # Count articles on a certain date
    def self.count_by_date(year, month = nil, day = nil, limit = nil)
      if !year.blank?
        count(:conditions => { :published_at => time_delta(year, month, day),
            :published => true })
      else
        count(:conditions => { :published => true })
      end
    end

    # Finds one article which was posted on a certain date and matches the supplied dashed-title
    # params is a Hash
    #    def self.find_by_permalink(params)
    #      date_range = self.time_delta(params[:year], params[:month], params[:day])
    #
    #      req_params = {}
    #      req_params[:permalink] = params[:title] if params[:title]
    #      req_params[:published_at] = date_range if date_range
    #
    #      return nil if req_params.empty? # no search if no params send
    #
    #      article = find_published(:first, :conditions => req_params)
    #      return article if article
    #
    #      if params[:title]
    #        req_params[:permalink] = CGI.escape(params[:title])
    #        article = find_published(:first, :conditions => req_params)
    #        return article if article
    #      end
    #
    #      raise ActiveRecord::RecordNotFound
    #    end

    def self.find_by_params_hash(params = {})
      params[:title] ||= params[:article_id]
      find_by_permalink(params)
    end

    # Fulltext searches the body of published articles
    def self.search(query, args={})
      query_s = query.to_s.strip
      if !query_s.empty? && args.empty?
        ::ContentManagement::Content.searchstring(query)
      elsif !query_s.empty? && !args.empty?
        ::ContentManagement::Content.searchstring(query).paginate(args)
      else
        []
      end
    end

    def add_comment(params)
      comments.build(params)
    end

    #---------------------------------------------------------------------------

    def type_is(content_type)
      self.content_type == ::ContentManagement::ContentType.find_by_name(content_type)
    end

    def format_is(content_format)
      self.content_format == ::ContentManagement::ContentFormat.find_by_name(content_format)
    end

    def published?
      !published_at.nil?
    end

    def pending?
      !published? || Time.now.utc < published_at
    end

    def status
      pending? ? :pending : :published
    end

    #---------------------------------------------------------------------------

    private

    def self.time_delta(year = nil, month = nil, day = nil)
      return nil if year.nil? && month.nil? && day.nil?
      from = Time.utc(year, month || 1, day || 1)

      to = from.next_year
      to = from.next_month unless month.blank?
      to = from + 1.day unless day.blank?
      to = to - 1 # pull off 1 second so we don't overlap onto the next day
      return from..to
    end

  end
end
